@@ -14,6 +14,18 @@
 #ifndef LLVM_CONFIG_H
 #define LLVM_CONFIG_H
 
+/*
+ * When bsd.lib.mk builds shared libraries it builds with -DPIC which 
+ * causes problems in the following files which use PIC as a variable name.
+ * undefine PIC here to minimise the diff to upstream LLVM
+ *
+ * include/llvm/MC/MCObjectFileInfo.h
+ * lib/MC/MCObjectFileInfo.cpp
+ * lib/Transforms/Scalar/LICM.cpp
+ * lib/Transforms/Utils/PredicateInfo.cpp
+ */
+#undef PIC
+
 /* Define if LLVM_ENABLE_DUMP is enabled */
 /* #undef LLVM_ENABLE_DUMP */
 
@@ -21,7 +33,7 @@
 /* #undef LINK_POLLY_INTO_TOOLS */
 
 /* Target triple LLVM will generate code for by default */
-#define LLVM_DEFAULT_TARGET_TRIPLE "armv7-unknown-netbsdelf8.99-eabihf"
+#define LLVM_DEFAULT_TARGET_TRIPLE "aarch64-unknown-openbsd6.7"
 
 /* Define if threads enabled */
 #define LLVM_ENABLE_THREADS 1
@@ -30,28 +42,28 @@
 #define LLVM_HAS_ATOMICS 1
 
 /* Host triple LLVM will be executed on */
-#define LLVM_HOST_TRIPLE "armv7-unknown-netbsdelf8.99-eabihf"
+#define LLVM_HOST_TRIPLE "aarch64-unknown-openbsd6.7"
 
 /* LLVM architecture name for the native architecture, if available */
-#define LLVM_NATIVE_ARCH ARM
+#define LLVM_NATIVE_ARCH AArch64
 
 /* LLVM name for the native AsmParser init function, if available */
-#define LLVM_NATIVE_ASMPARSER LLVMInitializeARMAsmParser
+#define LLVM_NATIVE_ASMPARSER LLVMInitializeAArch64AsmParser
 
 /* LLVM name for the native AsmPrinter init function, if available */
-#define LLVM_NATIVE_ASMPRINTER LLVMInitializeARMAsmPrinter
+#define LLVM_NATIVE_ASMPRINTER LLVMInitializeAArch64AsmPrinter
 
 /* LLVM name for the native Disassembler init function, if available */
-#define LLVM_NATIVE_DISASSEMBLER LLVMInitializeARMDisassembler
+#define LLVM_NATIVE_DISASSEMBLER LLVMInitializeAArch64Disassembler
 
 /* LLVM name for the native Target init function, if available */
-#define LLVM_NATIVE_TARGET LLVMInitializeARMTarget
+#define LLVM_NATIVE_TARGET LLVMInitializeAArch64Target
 
 /* LLVM name for the native TargetInfo init function, if available */
-#define LLVM_NATIVE_TARGETINFO LLVMInitializeARMTargetInfo
+#define LLVM_NATIVE_TARGETINFO LLVMInitializeAArch64TargetInfo
 
 /* LLVM name for the native target MC init function, if available */
-#define LLVM_NATIVE_TARGETMC LLVMInitializeARMTargetMC
+#define LLVM_NATIVE_TARGETMC LLVMInitializeAArch64TargetMC
 
 /* Define if this is Unixish platform */
 #define LLVM_ON_UNIX 1
@@ -72,10 +84,10 @@
 #define LLVM_VERSION_MINOR 0
 
 /* Patch version of the LLVM API */
-#define LLVM_VERSION_PATCH 0
+#define LLVM_VERSION_PATCH 1
 
 /* LLVM version string */
-#define LLVM_VERSION_STRING "8.0.0"
+#define LLVM_VERSION_STRING "8.0.1"
 
 /* Whether LLVM records statistics for use with GetStatistics(),
  * PrintStatistics() or PrintStatisticsJSON()
